@@ -4,19 +4,40 @@
 import queue
 import threading
 import datetime
+import hashlib
 import os
 import logging
 from GoogleScraper.utils import chunk_it
+from GoogleScraper.commandline import get_command_line
 from GoogleScraper.database import ScraperSearch, SERP, Link, get_session
-from GoogleScraper.proxies import parse_proxy_file, get_proxies_from_mysql_db
-from GoogleScraper.scraping import SelScrape, HttpScrape
-from GoogleScraper.caching import maybe_clean_cache, fix_broken_cache_names, _caching_is_one_to_one, parse_all_cached_files, clean_cachefiles
-from GoogleScraper.config import InvalidConfigurationException, parse_cmd_args, Config
+from GoogleScraper.proxies import parse_proxy_file, get_proxies_from_mysql_db, add_proxies_to_db
+from GoogleScraper.scraping import HttpScrape, get_selenium_scraper_by_search_engine_name
+from GoogleScraper.caching import fix_broken_cache_names, _caching_is_one_to_one, parse_all_cached_files, clean_cachefiles
+from GoogleScraper.config import InvalidConfigurationException, parse_cmd_args, Config, update_config_with_file
 from GoogleScraper.log import out
 import GoogleScraper.config
 
 logger = logging.getLogger('GoogleScraper')
 
+
+def id_for_keywords(keywords):
+    """Determine a unique id for the keywords.
+
+    Helps to continue the last scrape and to identify the last
+    scrape object.
+
+    Args:
+        keywords: All the keywords in the scrape process
+    Returns:
+        The unique md5 string of all keywords.
+    """
+
+    m = hashlib.md5()
+    for kw in keywords:
+        m.update(kw.encode())
+    return m.hexdigest()
+
+
 def scrape_with_config(config, **kwargs):
     """Runs GoogleScraper with the dict in config.
 
@@ -108,18 +129,28 @@ class ShowProgressQueue(threading.Thread):
         Args:
             queue: A queue.Queue instance to share among the worker threads.
             num_keywords: The number of total keywords that need to be scraped.
-
         """
         super().__init__()
         self.queue = queue
         self.num_keywords = num_keywords
         self.num_already_processed = 0
+        self.verbosity = Config['GLOBAL'].getint('verbosity', 1)
+        self.progress_fmt = '\033[92m{}/{} keywords processed.\033[0m'
 
     def run(self):
         while self.num_already_processed < self.num_keywords:
             e = self.queue.get()
+
+            if e == 'done':
+                break
+
             self.num_already_processed += 1
-            print('{}/{} keywords processed.'.format(self.num_already_processed, self.num_keywords), end='\r')
+
+            if self.verbosity == 1:
+                print(self.progress_fmt.format(self.num_already_processed, self.num_keywords), end='\r')
+            elif self.verbosity == 2 and self.num_already_processed % 5 == 0:
+                print(self.progress_fmt.format(self.num_already_processed, self.num_keywords))
+
             self.queue.task_done()
 
 
@@ -133,12 +164,18 @@ def main(return_results=False, parse_cmd_line=True):
     Args:
         return_results: When GoogleScrape is used from within another program, don't print results to stdout,
                         store them in a database instead.
+        parse_cmd_line: Whether to get options from the command line or not.
     Returns:
         A database session to the results when return_results is True
     """
     if parse_cmd_line:
         parse_cmd_args()
 
+    # If the configuration file to use is explicitly specified, update the current configuration
+    # with it.
+    if Config['GLOBAL'].get('config_file', None):
+        update_config_with_file(Config['GLOBAL'].get('config_file', None))
+
     if Config['GLOBAL'].getboolean('view_config'):
         from GoogleScraper.config import CONFIG_FILE
         print(open(CONFIG_FILE).read())
@@ -149,13 +186,12 @@ def main(return_results=False, parse_cmd_line=True):
         print(__version__)
         return
 
-    maybe_clean_cache()
-
-    kwfile = Config['SCRAPING'].get('keyword_file')
+    kwfile = Config['SCRAPING'].get('keyword_file', '')
     keyword = Config['SCRAPING'].get('keyword')
     keywords = {keyword for keyword in set(Config['SCRAPING'].get('keywords', []).split('\n')) if keyword}
     proxy_file = Config['GLOBAL'].get('proxy_file', '')
     proxy_db = Config['GLOBAL'].get('mysql_proxy_db', '')
+    num_search_engines = len(Config['SCRAPING'].get('search_engines', 1).split(','))
 
     if Config['GLOBAL'].getboolean('shell', False):
         namespace = {}
@@ -164,16 +200,19 @@ def main(return_results=False, parse_cmd_line=True):
         namespace['ScraperSearch'] = ScraperSearch
         namespace['SERP'] = SERP
         namespace['Link'] = Link
+        namespace['Proxy'] = GoogleScraper.database.Proxy
         print('Available objects:')
         print('session - A sqlalchemy session of the results database')
         print('ScraperSearch - Search/Scrape job instances')
         print('SERP - A search engine results page')
         print('Link - A single link belonging to a SERP')
+        print('Proxy - Proxies stored for scraping projects.')
         start_python_console(namespace)
         return
 
     if not (keyword or keywords) and not kwfile:
         logger.error('No keywords to scrape for. Please provide either an keyword file (Option: --keyword-file) or specify and keyword with --keyword.')
+        get_command_line(False, True)
         return
 
     if Config['GLOBAL'].getboolean('fix_cache_names'):
@@ -189,7 +228,8 @@ def main(return_results=False, parse_cmd_line=True):
             # Clean the keywords of duplicates right in the beginning
             keywords = set([line.strip() for line in open(kwfile, 'r').read().split('\n')])
 
-    search_engines = list({search_engine for search_engine in Config['SCRAPING'].get('search_engines', 'google').split(',') if search_engine})
+    search_engines = list({search_engine.strip() for search_engine
+        in Config['SCRAPING'].get('search_engines', 'google').split(',') if search_engine.strip()})
     assert search_engines, 'No search engine specified'
 
     if Config['GLOBAL'].getboolean('clean_cache_files', False):
@@ -239,12 +279,37 @@ def main(return_results=False, parse_cmd_line=True):
     Session = get_session(scoped=False, create=True)
     session = Session()
 
-    scraper_search = ScraperSearch(
-        number_search_engines_used=1,
-        number_proxies_used=len(proxies),
-        number_search_queries=len(keywords),
-        started_searching=datetime.datetime.utcnow()
-    )
+    # add proxies to the database
+    add_proxies_to_db(proxies, session)
+
+    # ask the user to continue the last scrape. We detect a continuation of a
+    # previously established scrape, if the keyword-file is the same and unmodified since
+    # the beginning of the last scrape.
+    scraper_search = None
+    if kwfile:
+        searches = session.query(ScraperSearch).\
+            filter(ScraperSearch.keyword_file == kwfile).\
+            order_by(ScraperSearch.started_searching).\
+            all()
+
+        if searches:
+            last_search = searches[-1]
+            last_modified = datetime.datetime.utcfromtimestamp(os.path.getmtime(last_search.keyword_file))
+
+            # if the last modification is older then the starting of the search
+            if last_modified < last_search.started_searching:
+                scraper_search = last_search
+                logger.info('Continuing last scrape.')
+
+    if not scraper_search:
+        scraper_search = ScraperSearch(
+            keyword_file=os.path.abspath(kwfile),
+            number_search_engines_used=num_search_engines,
+            number_proxies_used=len(proxies),
+            number_search_queries=len(keywords),
+            started_searching=datetime.datetime.utcnow(),
+            used_search_engines=','.join(search_engines)
+        )
 
     # First of all, lets see how many keywords remain to scrape after parsing the cache
     if Config['GLOBAL'].getboolean('do_caching'):
@@ -255,93 +320,96 @@ def main(return_results=False, parse_cmd_line=True):
     # remove duplicates and empty keywords
     remaining = [keyword for keyword in set(remaining) if keyword]
 
-    kwgroups = assign_keywords_to_scrapers(remaining)
+    if remaining:
 
-    # Create a lock to synchronize database access in the sqlalchemy session
-    db_lock = threading.Lock()
+        kwgroups = assign_keywords_to_scrapers(remaining)
 
-    # create a lock to cache results
-    cache_lock = threading.Lock()
+        # Create a lock to synchronize database access in the sqlalchemy session
+        db_lock = threading.Lock()
 
-    # final check before going into the loop
-    num_workers_to_allocate = len(kwgroups) * len(search_engines) > Config['SCRAPING'].getint('maximum_workers')
-    if (len(kwgroups) * len(search_engines)) > Config['SCRAPING'].getint('maximum_workers'):
-        logger.error('Too many workers: {} , might crash the app'.format(num_workers_to_allocate))
+        # create a lock to cache results
+        cache_lock = threading.Lock()
+
+        # final check before going into the loop
+        num_workers_to_allocate = len(kwgroups) * len(search_engines) > Config['SCRAPING'].getint('maximum_workers')
+        if (len(kwgroups) * len(search_engines)) > Config['SCRAPING'].getint('maximum_workers'):
+            logger.error('Too many workers: {} , might crash the app'.format(num_workers_to_allocate))
 
 
-    out('Going to scrape {num_keywords} keywords with {num_proxies} proxies by using {num_threads} threads.'.format(
-        num_keywords=len(remaining),
-        num_proxies=len(proxies),
-        num_threads=Config['SCRAPING'].getint('num_workers', 1)
-    ), lvl=1)
+        out('Going to scrape {num_keywords} keywords with {num_proxies} proxies by using {num_threads} threads.'.format(
+            num_keywords=len(remaining),
+            num_proxies=len(proxies),
+            num_threads=num_search_engines
+        ), lvl=1)
 
-    # Show the progress of the scraping
-    q = None
-    if Config['GLOBAL'].getint('verbosity', 1) == 1:
+        # Show the progress of the scraping
         q = queue.Queue()
         progress_thread = ShowProgressQueue(q, len(remaining))
         progress_thread.start()
 
-    # Let the games begin
-    if Config['SCRAPING'].get('scrapemethod') in ('selenium', 'http'):
-        # A lock to prevent multiple threads from solving captcha.
-        captcha_lock = threading.Lock()
+        # Let the games begin
+        if Config['SCRAPING'].get('scrapemethod') in ('selenium', 'http'):
+            # A lock to prevent multiple threads from solving captcha.
+            captcha_lock = threading.Lock()
 
-        # Distribute the proxies evenly on the keywords to search for
-        scrapejobs = []
+            # Distribute the proxies evenly on the keywords to search for
+            scrapejobs = []
 
-        for k, search_engine in enumerate(search_engines):
-            for i, keyword_group in enumerate(kwgroups):
-                
-                proxy_to_use = proxies[i % len(proxies)]
-                
-                if Config['SCRAPING'].get('scrapemethod', 'http') == 'selenium':
-                    scrapejobs.append(
-                        SelScrape(
-                            search_engine=search_engine,
-                            session=session,
-                            keywords=keyword_group,
-                            db_lock=db_lock,
-                            cache_lock=cache_lock,
-                            scraper_search=scraper_search,
-                            captcha_lock=captcha_lock,
-                            browser_num=i,
-                            proxy=proxy_to_use,
-                            progress_queue=q,
+            for k, search_engine in enumerate(search_engines):
+                for i, keyword_group in enumerate(kwgroups):
+
+                    proxy_to_use = proxies[i % len(proxies)]
+
+                    if Config['SCRAPING'].get('scrapemethod', 'http') == 'selenium':
+                        scrapejobs.append(
+                            get_selenium_scraper_by_search_engine_name(
+                                search_engine,
+                                search_engine=search_engine,
+                                session=session,
+                                keywords=keyword_group,
+                                db_lock=db_lock,
+                                cache_lock=cache_lock,
+                                scraper_search=scraper_search,
+                                captcha_lock=captcha_lock,
+                                browser_num=i,
+                                proxy=proxy_to_use,
+                                progress_queue=q
+                            )
                         )
-                    )
-                elif Config['SCRAPING'].get('scrapemethod') == 'http':
-                    scrapejobs.append(
-                        HttpScrape(
-                            search_engine=search_engine,
-                            keywords=keyword_group,
-                            session=session,
-                            scraper_search=scraper_search,
-                            cache_lock=cache_lock,
-                            db_lock=db_lock,
-                            proxy=proxy_to_use,
-                            progress_queue=q,
+                    elif Config['SCRAPING'].get('scrapemethod') == 'http':
+                        scrapejobs.append(
+                            HttpScrape(
+                                search_engine=search_engine,
+                                keywords=keyword_group,
+                                session=session,
+                                scraper_search=scraper_search,
+                                cache_lock=cache_lock,
+                                db_lock=db_lock,
+                                proxy=proxy_to_use,
+                                progress_queue=q,
+                            )
                         )
-                    )
 
-        for t in scrapejobs:
-            t.start()
+            for t in scrapejobs:
+                t.start()
 
-        for t in scrapejobs:
-            t.join()
+            for t in scrapejobs:
+                t.join()
 
-    elif Config['SCRAPING'].get('scrapemethod') == 'http-async':
-        raise NotImplemented('soon my dear friends :)')
+            # after threads are done, stop the progress queue.
+            q.put('done')
 
-    else:
-        raise InvalidConfigurationException('No such scrapemethod. Use "http" or "sel"')
+        elif Config['SCRAPING'].get('scrapemethod') == 'http-async':
+            raise NotImplemented('soon my dear friends :)')
 
-    scraper_search.stopped_searching = datetime.datetime.utcnow()
-    session.add(scraper_search)
-    session.commit()
+        else:
+            raise InvalidConfigurationException('No such scrapemethod. Use "http" or "selenium"')
+
+        scraper_search.stopped_searching = datetime.datetime.utcnow()
+        session.add(scraper_search)
+        session.commit()
 
-    if Config['GLOBAL'].getint('verbosity', 1) == 1:
         progress_thread.join()
 
-    if return_results:
-        return session
+        if return_results:
+            return session
