@@ -10,7 +10,6 @@ from urllib.parse import urlparse, unquote
 import pprint
 from GoogleScraper.database import SearchEngineResultsPage, Link
 from GoogleScraper.config import Config
-from GoogleScraper.log import out
 from cssselect import HTMLTranslator
 
 logger = logging.getLogger('GoogleScraper')
@@ -180,6 +179,8 @@ class Parser():
                                     pass
 
                         serp_result[key] = value
+                    serp_result['rank'] = index+1
+
                     # only add items that have not None links.
                     # Avoid duplicates. Detect them by the link.
                     # If statement below: Lazy evaluation. The more probable case first.
@@ -624,7 +625,7 @@ class BaiduParser(Parser):
 class DuckduckgoParser(Parser):
     """Parses SERP pages of the Duckduckgo search engine."""
     
-    search_types = ['normal', 'image']
+    search_types = ['normal']
     
     num_results_search_selectors = []
     
@@ -642,6 +643,49 @@ class DuckduckgoParser(Parser):
     }
 
 
+class AskParser(Parser):
+    """Parses SERP pages of the Ask search engine."""
+
+    search_types = ['normal']
+
+    num_results_search_selectors = []
+
+    normal_search_selectors = {
+        'results': {
+            'de_ip': {
+                'container': '#midblock',
+                'result_container': '.ptbs.ur',
+                'link': '.abstract > a::attr(href)',
+                'snippet': '.abstract::text',
+                'title': '.txt_lg.b::text',
+                'visible_link': '.durl span::text'
+            }
+        },
+    }
+
+
+class BlekkoParser(Parser):
+    """Parses SERP pages of the Blekko search engine."""
+
+    search_types = ['normal']
+
+    num_results_search_selectors = []
+
+    normal_search_selectors = {
+        'results': {
+            'de_ip': {
+                'container': '#links',
+                'result_container': '.result',
+                'link': '.result__title > a::attr(href)',
+                'snippet': 'result__snippet::text',
+                'title': '.result__title > a::text',
+                'visible_link': '.result__url__domain::text'
+            }
+        },
+    }
+
+
+
 def get_parser_by_url(url):
     """Get the appropriate parser by an search engine url.
 
@@ -668,7 +712,10 @@ def get_parser_by_url(url):
         parser = BaiduParser
     elif re.search(r'^https://duckduckgo\.com', url):
         parser = DuckduckgoParser
-
+    if re.search(r'^http[s]?://[a-z]{2}?\.ask', url):
+        parser = AskParser
+    if re.search(r'^http[s]?://blekko', url):
+        parser = BlekkoParser
     if not parser:
         raise UnknowUrlException('No parser for {}.'.format(url))
 
@@ -699,6 +746,10 @@ def get_parser_by_search_engine(search_engine):
         return BaiduParser
     elif search_engine == 'duckduckgo':
         return DuckduckgoParser
+    elif search_engine == 'ask':
+        return AskParser
+    elif search_engine == 'blekko':
+        return BlekkoParser
     else:
         raise NoParserForSearchEngineException('No such parser for {}'.format(search_engine))
 
@@ -720,7 +771,6 @@ def parse_serp(html=None, search_engine=None,
             parser = parser()
             parser.parse(html)
 
-        out(parser, lvl=2)
         num_results = 0
 
         if not serp:
@@ -736,24 +786,26 @@ def parse_serp(html=None, search_engine=None,
 
         for key, value in parser.search_results.items():
             if isinstance(value, list):
-                rank = 1
                 for link in value:
                     parsed = urlparse(link['link'])
 
                     # fill with nones to prevent key errors
                     [link.update({key: None}) for key in ('snippet', 'title', 'visible_link') if key not in link]
 
+
+                    # sqlalchemy will automatically add this link to
+                    # the serp object.
                     l = Link(
                         link=link['link'],
                         snippet=link['snippet'],
                         title=link['title'],
                         visible_link=link['visible_link'],
                         domain=parsed.netloc,
-                        rank=rank,
-                        serp=serp
+                        rank=link['rank'],
+                        serp=serp,
+                        link_type=key
                     )
                     num_results += 1
-                    rank += 1
 
         serp.num_results = num_results
 
