@@ -7,8 +7,11 @@ import math
 import logging
 import sys
 import time
+import os
 import socket
+import json
 import abc
+from urllib.parse import urlencode
 
 try:
     from selenium import webdriver
@@ -23,8 +26,8 @@ except ImportError as ie:
 
 import GoogleScraper.socks as socks
 from GoogleScraper.proxies import Proxy
-from GoogleScraper.caching import get_cached, cache_results, cached_file_name, cached
-from GoogleScraper.database import SearchEngineResultsPage, Link, get_session
+from GoogleScraper.caching import cache_results
+from GoogleScraper.database import SearchEngineResultsPage, db_Proxy
 from GoogleScraper.config import Config
 from GoogleScraper.log import out
 from GoogleScraper.output_converter import store_serp_result, end, dict_from_scraping_object
@@ -50,6 +53,25 @@ class SeleniumMisconfigurationError(Exception):
 class SeleniumSearchError(Exception):
     pass
 
+class StopScrapingException(Exception):
+    pass
+
+
+"""
+GoogleScraper should be as robust as possible.
+
+There are several conditions that may stop the scraping process. In such a case,
+a StopScrapingException is raised with the reason.
+
+- All proxies are detected and we cannot request further keywords => Stop.
+- No internet connection => Stop.
+
+- If the proxy is detected by the search engine we try to get another proxy from the pool and we call switch_proxy() => continue.
+
+- If the proxy is detected by the search engine and there is no other proxy in the pool, we wait {search_engine}_proxy_detected_timeout seconds => continue.
+    + If the proxy is detected again after the waiting time, we discard the proxy for the whole scrape.
+"""
+
 
 def get_base_search_url_by_search_engine(search_engine_name, search_mode):
     """Retrieves the search engine base url for a specific search_engine.
@@ -71,9 +93,16 @@ def get_base_search_url_by_search_engine(search_engine_name, search_mode):
     specific_base_url = Config[search_mode.upper()].get('{}_search_url'.format(search_engine_name), None)
 
     if not specific_base_url:
-        return Config['SCRAPING'].get('{}_search_url'.format(search_engine_name), None)
-    else:
-        return specific_base_url
+        specific_base_url = Config['SCRAPING'].get('{}_search_url'.format(search_engine_name), None)
+
+    ipfile = Config['SCRAPING'].get('{}_ip_file'.format(search_engine_name), '')
+
+    if os.path.exists(ipfile):
+        ips = open(ipfile, 'rt').read().split('\n')
+        random_ip = random.choice(ips)
+        return random_ip
+
+    return specific_base_url
 
     
 class SearchEngineScrape(metaclass=abc.ABCMeta):
@@ -109,6 +138,20 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
     sophisticated input format and some more tricky engineering.
     """
 
+
+    malicious_request_needles = {
+        'google': {
+            'inurl': '/sorry/',
+            'inhtml': 'detected unusual traffic'
+        },
+        'bing': {},
+        'yahoo': {},
+        'baidu': {},
+        'yandex': {},
+        'ask': {},
+        'blekko': {}
+    }
+
     def __init__(self, keywords=None, scraper_search=None, session=None, db_lock=None, cache_lock=None,
                  start_page_pos=1, search_engine=None, search_type=None, proxy=None, progress_queue=None):
         """Instantiate an SearchEngineScrape object.
@@ -139,6 +182,9 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
 
         self.keywords = list(set(self.keywords))
 
+        # the keywords that couldn't be scraped by this worker
+        self.missed_keywords = set()
+
         # the number of keywords
         self.num_keywords = len(self.keywords)
         
@@ -177,6 +223,9 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
         # the scrape mode
         # to be set by subclasses
         self.scrapemethod = ''
+        
+        # Whether the instance is ready to run
+        self.startable = True
 
         # set the database lock
         self.db_lock = db_lock
@@ -202,12 +251,17 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
             self.sleeping_ranges[int(key)] = tuple([int(offset.strip()) for offset in value.split(',')])
 
 
+        # the default timeout
+        self.timeout = 5
+
+
+
     @abc.abstractmethod
     def search(self, *args, **kwargs):
         """Send the search request(s) over the transport."""
 
 
-    def blocking_search(self, callback, *args, nextkw=None, **kwargs):
+    def blocking_search(self, callback, *args, **kwargs):
         """Similar transports have the same search loop layout.
 
         The SelScrape and HttpScrape classes have the same search loops. Just
@@ -216,16 +270,27 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
 
         Args:
             callback: A callable with the search functionality.
+            args: Arguments for the callback
+            kwargs: Keyword arguments for the callback.
         """
-        for self.current_keyword in self.keywords:
+        for i, self.current_keyword in enumerate(self.keywords):
 
             self.current_page = self.start_page_pos
 
             for self.current_page in range(1, self.num_pages_per_keyword + 1):
 
                 # set the actual search code in the derived class
-                callback(*args, **kwargs)
+                try:
 
+                    if not callback(*args, **kwargs):
+                        self.missed_keywords.add(self.current_keyword)
+
+                except StopScrapingException as e:
+                    # Leave search when search engines detected us
+                    # add the rest of the keywords as missed one
+                    logger.critical(e)
+                    self.missed_keywords.extend(self.keywords[i:])
+                    return
 
     @abc.abstractmethod
     def set_proxy(self):
@@ -240,18 +305,26 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
     def proxy_check(self, proxy):
         """Check whether the assigned proxy works correctly and react"""
 
-        
+
     @abc.abstractmethod
-    def handle_request_denied(self):
-        """Behaviour when search engines detect our scraping."""
+    def handle_request_denied(self, status_code):
+        """Generic behaviour when search engines detect our scraping.
+
+        Args:
+            status_code: The status code of the http response.
+        """
+        logger.warning('Malicious request detected: {}'.format(status_code))
+
+        # cascade
+        timeout = Config['PROXY_POLICY'].getint('{search_engine}_proxy_detected_timeout'.format(
+            search_engine=self.search_engine), Config['PROXY_POLICY'].getint('proxy_detected_timeout'))
+        time.sleep(timeout)
 
     def store(self):
         """Store the parsed data in the sqlalchemy scoped session."""
         assert self.session, 'No database session. Turning down.'
 
         with self.db_lock:
-            num_results = 0
-
             serp = SearchEngineResultsPage(
                 search_engine_name=self.search_engine,
                 scrapemethod=self.scrapemethod,
@@ -264,11 +337,17 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
             self.scraper_search.serps.append(serp)
 
             serp, parser = parse_serp(serp=serp, parser=self.parser)
-            self.session.add(serp)
-            self.session.commit()
+
+            if serp.num_results > 0:
+                self.session.add(serp)
+                self.session.commit()
+            else:
+                return False
 
             store_serp_result(dict_from_scraping_object(self), self.parser)
 
+            return True
+
     def next_page(self):
         """Increment the page. The next search request will request the next page."""
         self.start_page_pos += 1
@@ -288,8 +367,8 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
 
     def instance_creation_info(self, scraper_name):
         """Debug message whenever a scraping worker is created"""
-        out('[+] {}[{}][search-type:{}] created using the search engine {}. Number of keywords to scrape={}, using proxy={}, number of pages per keyword={}'.format(
-            scraper_name, self.ip, self.search_type, self.search_engine, len(self.keywords), self.proxy, self.num_pages_per_keyword), lvl=2)
+        out('[+] {}[{}][search-type:{}][{}] using search engine "{}". Num keywords ={}, num pages for keyword={}'.format(
+            scraper_name, self.ip, self.search_type, self.base_search_url, self.search_engine, len(self.keywords), self.num_pages_per_keyword), lvl=1)
 
 
     def cache_results(self):
@@ -323,21 +402,53 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
         self.current_delay = random.randrange(*self._largest_sleep_range(self.search_number))
         time.sleep(self.current_delay)
 
-    def after_search(self, html):
+    def after_search(self):
         """Store the results and parse em.
 
         Notify the progress queue if necessary.
-
-        Args:
-            html: The scraped html.
         """
-        self.parser.parse(html)
-        self.store()
+        self.parser.parse(self.html)
+        self.search_number += 1
+
+        if not self.store():
+            logger.error('No results to store, skip current keyword: "{0}"'.format(self.current_keyword))
+            return
+
         if self.progress_queue:
             self.progress_queue.put(1)
         self.cache_results()
-        self.search_number += 1
 
+    def before_search(self):
+        """Things that need to happen before entering the search loop."""
+
+        # check proxies first before anything
+        if Config['SCRAPING'].getboolean('check_proxies', True) and self.proxy:
+            if not self.proxy_check():
+                self.startable = False
+
+
+    def update_proxy_status(self, status, ipinfo={}, online=True):
+        """Sets the proxy status with the results of ipinfo.io
+
+        Args:
+            status: A string the describes the status of the proxy.
+            ipinfo: The json results from ipinfo.io
+            online: Whether the proxy is usable or not.
+        """
+
+        with self.db_lock:
+
+            proxy = self.session.query(db_Proxy).filter(self.proxy.host == db_Proxy.ip).first()
+            if proxy:
+                for key in ipinfo.keys():
+                    setattr(proxy, key, ipinfo[key])
+
+                proxy.checked_at = datetime.datetime.utcnow()
+                proxy.status = status
+                proxy.online = online
+
+                self.session.add(proxy)
+                self.session.commit()
 
     def __del__(self):
         """Close the json array if necessary."""
@@ -362,7 +473,7 @@ class HttpScrape(SearchEngineScrape, threading.Timer):
     # Several different User-Agents to diversify the requests.
     # Keep the User-Agents updated. Last update: 13th November 2014
     # Get them here: http://techblog.willshouse.com/2012/01/03/most-common-user-agents/
-    USER_AGENTS = [
+    user_agents = [
         'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10) AppleWebKit/600.1.25 (KHTML, like Gecko) Version/8.0 Safari/600.1.25',
         'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.2125.111 Safari/537.36',
         'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.2125.104 Safari/537.36',
@@ -415,10 +526,6 @@ class HttpScrape(SearchEngineScrape, threading.Timer):
         # get the base search url based on the search engine.
         self.base_search_url = get_base_search_url_by_search_engine(self.search_engine, self.scrapemethod)
 
-        # check proxies first before anything
-        if Config['SCRAPING'].getboolean('check_proxies'):
-            self.proxy_check()
-
         super().instance_creation_info(self.__class__.__name__)
 
 
@@ -450,17 +557,46 @@ class HttpScrape(SearchEngineScrape, threading.Timer):
     def proxy_check(self):
         assert self.proxy and self.requests, 'ScraperWorker needs valid proxy instance and requests library to make the proxy check.'
 
-        data = self.requests.get(Config['GLOBAL'].get('proxy_check_url')).text
+        online = False
+        status = 'Proxy check failed: {host}:{port} is not used while requesting'.format(**self.proxy.__dict__)
+        ipinfo = {}
 
-        if not self.proxy.host in data:
-            logger.warning('Proxy check failed: {host}:{port} is not used while requesting'.format(**self.proxy.__dict__))
+        try:
+            text = self.requests.get(Config['GLOBAL'].get('proxy_info_url')).text
+            try:
+                ipinfo = json.loads(text)
+            except ValueError as v:
+                pass
+        except self.requests.ConnectionError as e:
+            status = 'No connection to proxy server possible, aborting: {}'.format(e)
+        except self.requests.Timeout as e:
+            status = 'Timeout while connecting to proxy server: {}'.format(e)
+        except self.requests.exceptions.RequestException as e:
+            status = 'Unknown exception: {}'.format(e)
+
+        if 'ip' in ipinfo and self.proxy.host == ipinfo['ip']:
+            online = True
+            status = 'Proxy is working.'
         else:
-            logger.info('Proxy check successful: All requests going through {host}:{port}'.format(**self.proxy.__dict__))
+            logger.warning(status)
+
+        super().update_proxy_status(status, ipinfo, online)
+
+        return online
 
 
     def handle_request_denied(self, status_code):
-        super().handle_request_denied()
-        raise Exception('Request not allowed')
+        """Handle request denied by the search engine.
+
+        This is the perfect place to distinguish the different responses
+        if search engine detect exhaustive searching.
+
+        Args:
+            status_code: The status code of the HTTP response.
+
+        Returns:
+        """
+        super().handle_request_denied(status_code)
 
     def build_search(self):
         """Build the headers and params for the search request for the search engine."""
@@ -528,64 +664,119 @@ class HttpScrape(SearchEngineScrape, threading.Timer):
         elif self.search_engine == 'duckduckgo':
             self.parser = DuckduckgoParser()
             self.search_params['q'] = self.current_keyword
+        elif self.search_engine == 'ask':
+            self.search_params['q'] = self.current_keyword
+            self.search_params['qsrc'] = '0'
+            self.search_params['l'] = 'dir'
+            self.search_params['qo'] = 'homepageSearchBox'
+        elif self.search_engine == 'blekko':
+            self.search_params['q'] = self.current_keyword
             
     def search(self, *args, rand=False, **kwargs):
-        """The actual search for the search engine."""
+        """The actual search for the search engine.
+
+        When raising StopScrapingException, the scraper will stop.
+
+        When return False, the scraper tries to continue with next keyword.
+        """
 
         self.build_search()
 
         if rand:
-            self.headers['User-Agent'] = random.choice(self.USER_AGENTS)
+            self.headers['User-Agent'] = random.choice(self.user_agents)
 
         try:
-            out('[HTTP - {proxy}] Base_url: {base_url}, headers={headers}, params={params}'.format(
-                proxy=self.proxy,
-                base_url=self.base_search_url,
+            super().detection_prevention_sleep()
+            super().keyword_info()
+
+            request = self.requests.get(self.base_search_url + urlencode(self.search_params), headers=self.headers, timeout=5)
+
+            self.current_request_time = datetime.datetime.utcnow()
+            self.html = request.text
+
+            out('[HTTP - {url}, headers={headers}, params={params}'.format(
+                url=request.url,
                 headers=self.headers,
                 params=self.search_params),
             lvl=3)
 
-            super().detection_prevention_sleep()
-            super().keyword_info()
-
-            request = self.requests.get(self.base_search_url, headers=self.headers,
-                             params=self.search_params, timeout=5)
-
-            self.current_request_time = datetime.datetime.utcnow()
-
         except self.requests.ConnectionError as ce:
-            logger.error('Network problem occurred {}'.format(ce))
-            raise ce
+            reason = 'Network problem occurred {}'.format(ce)
+            raise StopScrapingException('Stopping scraping because {}'.format(reason))
         except self.requests.Timeout as te:
-            logger.error('Connection timeout {}'.format(te))
-            raise te
+            reason = 'Connection timeout {}'.format(te)
+            raise StopScrapingException('Stopping scraping because {}'.format(reason))
+        except self.requests.exceptions.RequestException as e:
+            # In case of any http networking exception that wasn't caught
+            # in the actual request, just end the worker.
+            raise StopScrapingException('Stopping scraping because {}'.format(e))
 
         if not request.ok:
-            logger.error('HTTP Error: {}'.format(request.status_code))
             self.handle_request_denied(request.status_code)
             return False
 
-        super().after_search(request.text)
+        super().after_search()
+
+        return True
 
     def run(self):
-        args = []
-        kwargs = {}
-        kwargs['rand'] = False
-        SearchEngineScrape.blocking_search(self, self.search, *args, **kwargs)
+        super().before_search()
 
-
-class AsyncHttpScrape(SearchEngineScrape):
-    pass
+        if self.startable:
+            args = []
+            kwargs = {}
+            kwargs['rand'] = False
+            SearchEngineScrape.blocking_search(self, self.search, *args, **kwargs)
 
 
 class SelScrape(SearchEngineScrape, threading.Thread):
-    """Instances of this class make use of selenium browser objects to query Google.
-
-    This is a quite cool approach if you ask me :D
+    """Instances of this class make use of selenium browser objects to query the search engines on a high level.
     """
 
+    next_page_selectors = {
+        'google': '#pnnext',
+        'yandex': '.pager__button_kind_next',
+        'bing': '.sb_pagN',
+        'yahoo': '#pg-next',
+        'baidu': '.n',
+        'ask': '#paging div a.txt3.l_nu'
+    }
+
+    input_field_selectors = {
+        'google': (By.NAME, 'q'),
+        'yandex': (By.NAME, 'text'),
+        'bing': (By.NAME, 'q'),
+        'yahoo': (By.NAME, 'p'),
+        'baidu': (By.NAME, 'wd'),
+        'duckduckgo': (By.NAME, 'q'),
+        'ask': (By.NAME, 'q'),
+        'blekko': (By.NAME, 'q'),
+    }
+
+    normal_search_locations = {
+        'google': 'https://www.google.com/',
+        'yandex': 'http://www.yandex.ru/',
+        'bing': 'http://www.bing.com/',
+        'yahoo': 'https://yahoo.com/',
+        'baidu': 'http://baidu.com/',
+        'duckduckgo': 'https://duckduckgo.com/',
+        'ask': 'http://ask.com/',
+        'blekko': 'http://blekko.com/'
+    }
+
+    image_search_locations = {
+        'google': 'https://www.google.com/imghp',
+        'yandex': 'http://yandex.ru/images/',
+        'bing': 'https://www.bing.com/?scope=images',
+        'yahoo': 'http://images.yahoo.com/',
+        'baidu': 'http://image.baidu.com/',
+        'duckduckgo': None, # duckduckgo doesnt't support direct image search
+        'ask': 'http://www.ask.com/pictures/',
+        'blekko': None,
+    }
+
     def __init__(self, *args, captcha_lock=None, browser_num=1, **kwargs):
-        """Create a new SelScraper Thread.
+        """Create a new SelScraper thread Instance.
 
         Args:
             captcha_lock: To sync captcha solving (stdin)
@@ -612,18 +803,31 @@ class SelScrape(SearchEngineScrape, threading.Thread):
     def switch_proxy(self, proxy):
         """Switch the proxy on the communication channel."""
 
-
     def proxy_check(self):
         assert self.proxy and self.webdriver, 'Scraper instance needs valid webdriver and proxy instance to make the proxy check'
 
-        self.webdriver.get(Config['GLOBAL'].get('proxy_check_url'))
+        online = False
+        status = 'Proxy check failed: {host}:{port} is not used while requesting'.format(**self.proxy.__dict__)
+        ipinfo = {}
 
-        data = self.webdriver.page_source
+        try:
+            self.webdriver.get(Config['GLOBAL'].get('proxy_info_url'))
+            try:
+                ipinfo = json.loads(self.webdriver.page_source)
+            except ValueError as v:
+                pass
+        except Exception as e:
+            status = str(e)
 
-        if not self.proxy.host in data:
-            logger.warning('Proxy check failed: {host}:{port} is not used while requesting'.format(**self.proxy.__dict__))
+        if 'ip' in ipinfo and self.proxy.host == ipinfo['ip']:
+            online = True
+            status = 'Proxy is working.'
         else:
-            logger.info('Proxy check successful: All requests going through {host}:{port}'.format(**self.proxy.__dict__))
+            logger.warning(status)
+
+        super().update_proxy_status(status, ipinfo, online)
+
+        return online
 
     def _get_webdriver(self):
         """Return a webdriver instance and set it up with the according profile/ proxies.
@@ -718,18 +922,7 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         Raises:
             MaliciousRequestDetected when there was not way to stp Google From denying our requests.
         """
-        malicious_request_needles = {
-            'google': {
-                'inurl': '/sorry/',
-                'inhtml': 'detected unusual traffic'
-            },
-            'bing': {},
-            'yahoo': {},
-            'baidu': {},
-            'yandex': {},
-        }
-
-        needles = malicious_request_needles[self.search_engine]
+        needles = self.malicious_request_needles[self.search_engine]
 
         if needles and needles['inurl'] in self.webdriver.current_url and needles['inhtml'] in self.webdriver.page_source:
 
@@ -743,7 +936,8 @@ class SelScrape(SearchEngineScrape, threading.Thread):
                     solution = input('enter the captcha please...')
                     self.webdriver.find_element_by_name('submit').send_keys(solution + Keys.ENTER)
                     try:
-                        self.search_input = WebDriverWait(self.webdriver, 5).until(self._get_search_input_field())
+                        self.search_input = WebDriverWait(self.webdriver, 5).until(
+                                EC.visibility_of_element_located(self._get_search_input_field()))
                     except TimeoutException as e:
                         raise MaliciousRequestDetected('Requesting with this ip is not possible at the moment.')
                     tf.close()
@@ -761,34 +955,12 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         """Build the search for SelScrapers"""
         assert self.webdriver, 'Webdriver needs to be ready to build the search'
 
-        # do the proxy check
-        if Config['SCRAPING'].getboolean('check_proxies'):
-            self.proxy_check()
-
-        normal_search_locations = {
-            'google': 'https://www.google.com/',
-            'yandex': 'http://www.yandex.ru/',
-            'bing': 'http://www.bing.com/',
-            'yahoo': 'https://yahoo.com/',
-            'baidu': 'http://baidu.com/',
-            'duckduckgo': 'https://duckduckgo.com/'
-        }
-
-        image_search_locations = {
-            'google': 'https://www.google.com/imghp',
-            'yandex': 'http://yandex.ru/images/',
-            'bing': 'https://www.bing.com/?scope=images',
-            'yahoo': 'http://images.yahoo.com/',
-            'baidu': 'http://image.baidu.com/',
-            'duckduckgo': None # duckduckgo doesnt't support direct image search
-        }
-
         self.starting_point = None
 
         if Config['SCRAPING'].get('search_type', 'normal') == 'image':
-            self.starting_point = image_search_locations[self.search_engine]
+            self.starting_point = self.image_search_locations[self.search_engine]
         else:
-            self.starting_point = normal_search_locations[self.search_engine]
+            self.starting_point = self.base_search_url
 
         self.webdriver.get(self.starting_point)
 
@@ -799,17 +971,7 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         Returns:
             A tuple to locate the search field as used by seleniums function presence_of_element_located()
         """
-
-        input_field_selectors = {
-            'google': (By.NAME, 'q'),
-            'yandex': (By.NAME, 'text'),
-            'bing': (By.NAME, 'q'),
-            'yahoo': (By.NAME, 'p'),
-            'baidu': (By.NAME, 'wd'),
-            'duckduckgo': (By.NAME, 'q')
-        }
-
-        return input_field_selectors[self.search_engine]
+        return self.input_field_selectors[self.search_engine]
 
     def _wait_until_search_input_field_appears(self, max_wait=5):
         """Waits until the search input field can be located for the current search engine
@@ -834,24 +996,14 @@ class SelScrape(SearchEngineScrape, threading.Thread):
             The href attribute of the next_url for further results.
         """
         if self.search_type == 'normal':
-            next_page_selectors = {
-                'google': '#pnnext',
-                'yandex': '.pager__button_kind_next',
-                'bing': '.sb_pagN',
-                'yahoo': '#pg-next',
-                'baidu': '.n',
-                'duckduckgo': '' # loads results dynamically with ajax
-            }
-
-            selector = next_page_selectors[self.search_engine]
-            next_url = ''
+            selector = self.next_page_selectors[self.search_engine]
             try:
                 # wait until the next page link emerges
                 WebDriverWait(self.webdriver, 5).until(EC.presence_of_element_located((By.CSS_SELECTOR, selector)))
                 element = self.webdriver.find_element_by_css_selector(selector)
                 next_url = element.get_attribute('href')
                 element.click()
-                self.current_request_time = datetime.datetime.utcnow()
+                return next_url
             except TimeoutException as te:
                 logger.warning('Cannot locate next page element: {}'.format(te))
                 return False
@@ -859,19 +1011,24 @@ class SelScrape(SearchEngineScrape, threading.Thread):
                 logger.warning('Cannot locate next page element: {}'.format(e))
                 return False
 
-            return next_url
         elif self.search_type == 'image':
             self.page_down()
             return True
 
+    def wait_until_serp_loaded(self):
+        # Waiting until the keyword appears in the title may
+        # not be enough. The content may still be from the old page.
+        try:
+            WebDriverWait(self.webdriver, 5).until(EC.title_contains(self.current_keyword))
+        except TimeoutException as e:
+            logger.error(SeleniumSearchError('Keyword "{}" not found in title: {}'.format(self.current_keyword, self.webdriver.title)))
+
     def search(self):
         """Search with webdriver.
 
         Fills out the search form of the search engine for each keyword.
         Clicks the next link while num_pages_per_keyword is not reached.
         """
-        n = 0
-
         for self.current_keyword in self.keywords:
 
             self.search_input = self._wait_until_search_input_field_appears()
@@ -885,55 +1042,54 @@ class SelScrape(SearchEngineScrape, threading.Thread):
                 self.search_input.send_keys(self.current_keyword + Keys.ENTER)
                 self.current_request_time = datetime.datetime.utcnow()
             else:
-                raise GoogleSearchError('Cannot get handle to the input form!')
+                logger.warning('Cannot get handle to the input form for keyword {}.'.format(self.current_keyword))
+                continue
 
             super().detection_prevention_sleep()
             super().keyword_info()
 
             for self.current_page in range(1, self.num_pages_per_keyword + 1):
-                # Waiting until the keyword appears in the title may
-                # not be enough. The content may still be from the old page.
-                try:
-                    WebDriverWait(self.webdriver, 5).until(EC.title_contains(self.current_keyword))
-                except TimeoutException as e:
-                    logger.error(SeleniumSearchError('Keyword "{}" not found in title: {}'.format(self.current_keyword, self.webdriver.title)))
-                    break
 
-                super().after_search(self.webdriver.page_source)
+                self.wait_until_serp_loaded()
+
+                try:
+                    self.html = self.webdriver.execute_script('return document.body.innerHTML;')
+                except WebDriverException as e:
+                    self.html = self.webdriver.page_source
+
+                super().after_search()
 
                 # Click the next page link not when leaving the loop
+                # in the next iteration.
                 if self.current_page < self.num_pages_per_keyword:
                     self.next_url = self._goto_next_page()
-
+                    self.current_request_time = datetime.datetime.utcnow()
+                    
                     if not self.next_url:
                         break
 
-            n += 1
-
     def page_down(self):
         """Scrolls down a page with javascript.
 
-        Used for next page in image search mode.
+        Used for next page in image search mode or when the 
+        next results are obtained by scrolling down a page.
         """
         js = '''
-        var B = document.body,
-            H = document.documentElement,
-            height
+        var w = window,
+            d = document,
+            e = d.documentElement,
+            g = d.getElementsByTagName('body')[0],
+            y = w.innerHeight|| e.clientHeight|| g.clientHeight;
 
-        if (typeof document.height !== 'undefined') {
-            height = document.height // For webkit browsers
-        } else {
-            height = Math.max( B.scrollHeight, B.offsetHeight,H.clientHeight, H.scrollHeight, H.offsetHeight );
-        }
-
-        window.scrollBy(0,800)
+        window.scrollBy(0,y);
+        return y;
         '''
 
         self.webdriver.execute_script(js)
-        time.sleep(1.5)
 
     def run(self):
         """Run the SelScraper."""
+
         if not self._get_webdriver():
             raise SeleniumMisconfigurationError('Aborting due to no available selenium webdriver.')
 
@@ -943,8 +1099,108 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         except WebDriverException as e:
             logger.error(e)
 
-        self.build_search()
+        super().before_search()
 
-        self.search()
+        if self.startable:
+            self.build_search()
+            self.search()
 
-        self.webdriver.close()
+        if self.webdriver:
+            self.webdriver.close()
+
+
+"""
+For most search engines, the normal SelScrape works perfectly, but sometimes
+the scraping logic is different for other search engines.
+
+Duckduckgo loads new results on the fly (via ajax) and doesn't support any "next page"
+link. Other search engines like gekko.com have a completely different SERP page format.
+
+That's why we need to inherit from SelScrape for specific logic that only applies for the given
+search engine.
+
+The following functionality may differ in particular:
+
+    - _goto_next_page()
+    - _get_search_input()
+    - _wait_until_search_input_field_appears()
+    - _handle_request_denied()
+    - wait_until_serp_loaded()
+"""
+
+class DuckduckgoSelScrape(SelScrape):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self.largest_id = 0
+
+    def _goto_next_page(self):
+        super().page_down()
+        return 'No more results' not in self.html
+    
+    def wait_until_serp_loaded(self):
+        def new_results(driver):
+            try:
+                elements = driver.find_elements_by_css_selector('[id*="r1-"]')
+                if elements:
+                    i = sorted([int(e.get_attribute('id')[3:]) for e in elements])[-1]
+                    return i > self.largest_id
+                else:
+                    return False
+            except WebDriverException:
+                pass
+
+        try:
+            WebDriverWait(self.webdriver, 5).until(new_results)
+        except TimeoutException as e:
+            pass
+
+        elements = self.webdriver.find_elements_by_css_selector('[id*="r1-"]')
+        try:
+            self.largest_id = sorted([int(e.get_attribute('id')[3:]) for e in elements])[-1]
+        except:
+            self.largest_id = 0
+        
+
+class BlekkoSelScrape(SelScrape):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        
+    def _goto_next_page(self):
+        pass
+
+class AskSelScrape(SelScrape):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def wait_until_serp_loaded(self):
+        
+        def wait_until_keyword_in_url(driver):
+            try:
+                return self.current_keyword in driver.current_url
+            except WebDriverException as e:
+                pass
+            
+        WebDriverWait(self.webdriver, 5).until(wait_until_keyword_in_url)
+
+
+def get_selenium_scraper_by_search_engine_name(search_engine_name, *args, **kwargs):
+    """Get the appropriate selenium scraper for the given search engine name.
+
+    Args:
+        search_engine_name: The search engine name.
+        args: The arguments for the target search engine instance creation.
+        kwargs: The keyword arguments for the target search engine instance creation.
+    Returns;
+        Either a concrete SelScrape instance specific for the given search engine or the abstract SelScrape object.
+    """
+    class_name = search_engine_name[0].upper() + search_engine_name[1:].lower() + 'SelScrape'
+    ns = globals()
+    if class_name in ns:
+        return ns[class_name](*args, **kwargs)
+
+    return SelScrape(*args, **kwargs)
+
+
+class AsyncHttpScrape(SearchEngineScrape):
+    pass
